#!/bin/bash

echo "********************************************************************************"
echo "* IMPORTANT: THIS SCRIPT WILL DELETE THE STORAGE/ DIRECTORY AND THE DATABASE   *"
echo "* USED BY THE LARDER.WEB WEBSITE. MAKE SURE YOU UNDERSTAND WHAT THIS SCRIPT    *"
echo "* DOES BEFORE RUNNING IT. ALSO, MAKE SURE YOU HAVE A VALID SNAPSHOT TO REPLACE *"
echo "* THE PREVIOUS VERSION OF THE STORAGE/ DIRECTORY AND THE DATABASE.             *"
echo "********************************************************************************"

function ConfirmServiceRunning() {
    while true; do
        read -p "Are you running the website as a service? (YES/NO) " yn
        case $yn in
            [Yy]* ) RunningAsService=0;
                    return 0;;
            [Nn]* ) RunningAsService=1;
                    return 1;;
            [Cc]* ) exit;;
            * ) echo "Please answer YES or NO.";;
        esac
    done
}

read -p "Snapshot Directory: " SnapshotDir
read -p "Database Name: " DatabaseName
read  -s -p "Sudo Password: " SudoPassword

echo ""
echo "INFO: If you need to stop the website manually, and haven't, press C to stop this script."

ConfirmServiceRunning

if [[ $RunningAsService -eq 0 ]]; then
    # Stop the website service, so you can restore from the
    # #restore-from-snapshot. Make sure the service name you've used matches the one
    # in the line below.
    echo "INFO: Stopping website service..."
    echo "$SudoPassword" | sudo -S systemctl stop larder.web.service
else
    echo "INFO: Skipping attempt to stop website service. Manual stoppage expected."
fi

# Delete the old storage/ directory and replace it with the one within the
# snapshot directory.
echo "INFO: Deleting old storage/ directory..."
rm -r ../storage/
echo "INFO: Replacing old storage/ directory with snapshot version..."
cp -r ../snapshot/$SnapshotDir/storage/ ../

# Drop the previous version of the database and replace it with the dump file
# stored within the snapshot directory.
echo "INFO: Dropping old database..."
echo "$SudoPassword" | sudo -S -u postgres psql -c "DROP DATABASE $DatabaseName WITH (FORCE);"
echo "INFO: Creating new database using snapshot dump file..."
echo "$SudoPassword" | sudo -S -u postgres psql -c "CREATE DATABASE $DatabaseName WITH OWNER larder"
DumpFile=$(ls ../snapshot/$SnapshotDir/*.sql)
# echo "INFO: Dump File -- $DumpFile"
psql -h localhost -p 5432 -U larder -d larderdev -f $DumpFile

if [[ $RunningAsService -eq 0 ]]; then
    # Restart the website service using the data from the snapshot.
    echo "INFO: Restarting website service..."
    echo "$SudoPassword" | sudo -S systemctl start larder.web.service
else
    echo "INFO: Skipping service restart. Manual restart expected."
fi
