#| Snippets to use in SLIME
================================================================================
These snippets are mostly to reduce the amount of typing you need to do when
entering stuff into the SLIME REPL. The biggest annoyance with the SLIME REPL is
the need to reference full package names when calling functions, causing a lot
of extra typing. For example, (larder.web:start :server 'woo) becomes, (start).

These functions are outside the project .asd file. Therefore, you will need to
evaluate them separately before you can use them in the SLIME REPL. You can,
also, C-c C-y to add function (at point) directly into the SLIME REPL. Again,
reducing the amount of typing needed.
|#

(defun start ()
  "Quicker way to start the server."
  (larder.web:start :server 'woo))

(defun stop ()
  "Quicker way to stop the server."
  (larder.web:stop))

(defun change-to (app-env)
  "Quickly change the app-environment to `APP-ENV', use 'dev', 'test' or 'prod'.
Note the use of the ~/.pgpass file. This is so the system has root access to
postgres and it's various operations/featture-set. The most notable is the
ability to dump a database using pg_dump whilst using the website -- so no
dev. logged into the machine via SSH."
  (cond ((string= "dev" app-env)
         (larder.web.config:change-app-env :dev t)
         (storage:store-text-file
          "~/.pgpass" (format nil "localhost:5432:~a:~a:~a"
                             (larder.web.config:database-name)
                             (larder.web.config:database-username)
                             (larder.web.config:database-password))))
        ((string= "test" app-env)
         (larder.web.config:change-app-env :test t)
         (storage:store-text-file
          "~/.pgpass" (format nil "localhost:5432:~a:~a:~a"
                             (larder.web.config:database-name)
                             (larder.web.config:database-username)
                             (larder.web.config:database-password))))
        ((string= "prod" app-env)
         (larder.web.config:change-app-env :prod t)
         (storage:store-text-file
          "~/.pgpass" (format nil "localhost:5432:~a:~a:~a"
                             (larder.web.config:database-name)
                             (larder.web.config:database-username)
                             (larder.web.config:database-password))))
        (t (format t "No enviroment provided. Defaulting to 'dev' enviroment.")
           (larder.web.config:change-app-env :dev t)
           (storage:store-text-file
            "~/.pgpass" (format nil "localhost:5432:~a:~a:~a"
                               (larder.web.config:database-name)
                               (larder.web.config:database-username)
                               (larder.web.config:database-password))))))

(defun env ()
  "Get the current application environment (app-env)."
  (larder.web.config:appenv))

(defun qld ()
  "Quickload this project."
  (ql:quickload :larder.web))

(defun qdel ()
  "Delete the project's FASLS files."
  (swank:delete-system-fasls "larder.web"))

(defun migrate-all ()
  "Migrates the database tables."
  (larder.web.db:migrate-all))

(defun reset-website ()
  "Deletes all the files in storage/ and drops all tables in the database."
  (larder.web.db:drop-all-tables)
  (storage:clear-storage)
  (larder.web.db:ensure-tables-exist))
