(defpackage #:auth
  (:use #:cl
	#:hermetic
	#:sxql
	#:ningle
	#:mito
	#:app-constants)
  (:import-from #:larder.web.db
		#:connection-settings
		#:db
		#:with-connection)
  (:import-from #:models #:account)
  (:export #:csrf-token
	   #:get-user-roles
	   #:get-current-user
           #:get-account-db-id
	   #:auth-user-data
           #:get-account-id))
(in-package #:auth)

(defun csrf-token ()
  "Cross-Site Request Forgery (CSRF) token."
  (cdr (assoc "lack.session"
	      (lack.request:request-cookies ningle:*request*)
	      :test #'string=)))

(hermetic:setup
 ;; #' is needed. (hermetic:roles) generates infinite-loop when called
 ;; otherwise -- 'roles' called in other parts of code-base.
 :user-p #'(lambda (username)
	     (with-connection (db)
	       (mito:find-dao 'models:account :username username)))
 :user-pass #'(lambda (username)
		(models::account-password
		 (with-connection (db)
		   (mito:find-dao 'models:account :username username))))
 :user-roles #'(lambda (username)
		 (let ((user
                         (with-connection (db)
                           (mito:find-dao 'models:account :username username))))
                   (list
                    (and user
			 (string-equal (models::account-account-type user) "user")
			 ':user)
		    (and user
			 (string-equal (models::account-account-type user) "administrator")
			 ':administrator)
                    (and user
                         (string-equal (models::account-account-type user) "owner")
                         ':owner))))
 :session ningle:*session*
 :denied (constantly '(400 (:content-type "text/plain") ("Authentication denied"))))

(defun get-current-user()
  "Returns the currently logged in user from the browser session."
  (with-connection (db)
    (mito:find-dao 'models:account :id (gethash :id ningle:*session*))))

(defun get-account-db-id (username)
  "Returns the Id. number of the specified `USERNAME' in the database.
This refers to the the database Id. and not the `ACCOUNT-ID' associated with the
`ACCOUNT' class."
  (with-connection (db)
    (mito:object-id
     (mito:find-dao 'models:account :username username))))

(defun get-account-id (username)
  "Returns the `ACCOUNT-ID' from the `ACCOUNT' with the specified `USERNAME'.
This is a UUID and is part of the `ACCOUNT' class. This is NOT the database
Id. (something called the row Id.). This `ACCOUNT-ID' should be viewed as a more
sticky Id. for a user's actual account."
  (models::account-id-of
   (with-connection (db) (mito:find-dao 'models:account :username username))))

(defun auth-user-data ()
  "Get usual session data for authorised user.
This data is used across most routes and acts as a helper function more than
anything else. Reduces the amount of typing you need to do in the `RENDER'
functions of each route in the `WEB' packages."
  `(:token ,(auth:csrf-token)
    :roles ,(auth:get-user-roles)
    :user ,(auth:get-current-user)))

(defun get-user-roles()
  "Returns a list of roles the current user has assigned to them.
This is mostly to check if the user is logged-in or has administration
privileges. You can then create if-blocks in the HTML templates and
control what the user can and cannot see or do."
  ;; (format t "~&[ROLES] ~a" (hermetic:roles))
  (loop :for role :in (hermetic:roles)
	:collect role
	:collect t))
