(defpackage #:models
  (:use #:cl
        #:app-constants
        #:mito)
  (:export #:account
           #:page
           #:recipe
           #:site_settings
           #:user_recipe
           #:file
           #:favourite))
(in-package #:models)

#| NOTE: If using SQLite use `APP-CONSTANTS' package.
================================================================================
As far as I am aware, SQLite can't store Boolean values. To get around this, it
is common to see 0 and 1 as sentinel values.

0 = app-constants:+false+
1 = app-constants:+true+

If you are using SQLite, use the `+TRUE+' and `+FALSE+' constants instead of
hard coding 0 and 1 into the Boolean base slots below. If the website is using
something like MySQL or PostgreSQL, you don't nned to worry about this. Just use
the Boolean values as intended.
|#

#| NOTE: The use of mito:deftable instead of defclass.
================================================================================
More information about mito: https://github.com/fukamachi/mito/

This code-base uses mito, which is an ORM. To define the classes for the
database, mito has a `DEFTABLE' macro. You can use `DEFCLASS' if you
prefer. `DEFTABLE' reduces on typing and that is why it's used here.

One of features of the `DEFTABLE' macro is the built-in `:ACCESSOR' slot. To
access slots, follow the naming convention: `<CLASS-NAME>-'. An example of this
would be, `ACCOUNT-USERNAME' of `ACCOUNT-ACTIVE'. An expanded example would be,

(account-active *user*)

This would return the `ACTIVE' slot from the `*USER*' parameter, which is an
instance of the `ACCOUNT' class.
|#

(mito:deftable account ()
  ((account-id   :col-type :uuid)
   (username     :col-type :text)
   (display-name :col-type :text)
   (password     :col-type :text)
   (account-type :col-type :text)
   (active       :col-type :boolean)))

(mito:deftable recipe ()
  ((title        :col-type   :text)
   (recipe-id    :col-type   :uuid)
   (author-id    :references (account account-id))
   (source       :col-type   (or :text :null))
   (ingredients  :col-type   (or :text :null)) ; Raw string for search database.
   (thumbnail-id :col-type   (or :uuid :null))
   (public       :col-type   :boolean)))

(mito:deftable site_settings ()
  ;; Breaks naming convention to match database name. 'site-settings' isn't used
  ;; as database name 'site_settings' is.
  ((allow-sign-up    :col-type :boolean)
   (show-log-in      :col-type :boolean)
   (show-site-logo   :col-type :boolean)
   (site-name        :col-type (or :text :null))
   (search-url       :col-type (or :text :null))
   (search-index     :col-type (or :text :null))
   (search-api-key   :col-type (or :text :null))
   (show-search      :col-type :boolean)))

(mito:deftable page ()
  ((title           :col-type   (or :text :null))
   (owner           :references (account account-id))
   (page-id         :col-type   :uuid)
   (enable-nav-menu :col-type   :boolean)
   (can-delete      :col-type   :boolean)
   (public          :col-type   :boolean)))

(mito:deftable file ()
  ((title        :col-type   :text)
   (mime         :col-type   :text)
   (account-id   :references (account account-id))
   (thumbnail-id :col-type   (or :text :null))
   (public       :col-type   :boolean)
   (file-id      :col-type   :uuid)))

(mito:deftable user_recipe ()
  ;; Breaks naming convention to match database name. 'user-recipe' isn't used
  ;; as database name.
  ((account-id     :references (account account-id))
   (public         :col-type   :boolean)
   (title          :col-type   :text)
   (user-recipe-id :col-type   :uuid)))

(mito:deftable favourite ()
  ((recipe-id :references (recipe recipe-id))
   (account-id :references (account account-id))))
