(in-package #:cl-user)
(defpackage #:serial
  (:use #:cl
        #:models
        #:com.inuoe.jzon)
  (:local-nicknames (#:jzon #:com.inuoe.jzon)) 
  (:export #:recipe-obj-to-json
           #:recipe-to-json))
(in-package #:serial)

#| Note on Serialisation Approaches
================================================================================
You will find most of the functions in here are for transforming and serialising
parts of objects/data. If you want to just convert an entire object, you can run
something like,

(com.inuoe.jzon:stringify
    (larder.web.db:get-recipe :recipe-id "cd9bdc4d-3bdc-5d82-b213-f2d36134ed84")
     :pretty t :stream t)

and the whole object is converted to a JSON object (as a string). There is very
little reason to write a wrapper for that type of code. So, you will be better
calling that directly in any of the routes which require serialised data.
|#

(defun recipe-obj-to-json (recipe)
  "Takes a `RECIPE' object and serialises it to a JSON object, for Meilisearch.
The `RECIPE' class has slots which are not relevant/needed/part of the
Meilisearch instance. So, this function pulls out the relevant parts and
converts them to JSON."
  (with-output-to-string (s)
    (jzon:with-writer* (:stream s :pretty t)
      (jzon:with-object*
        (jzon:write-key* :recipe-id)    (jzon:write-value* (models::recipe-recipe-id recipe))
        (jzon:write-key* :title)        (jzon:write-value* (models::recipe-title recipe))
        (jzon:write-key* :thumbnail-id) (jzon:write-value* (models::recipe-thumbnail-id recipe))
        (jzon:write-key* :ingredients)  (jzon:write-value* (models::recipe-ingredients recipe))
        (jzon:write-key* :source)
        (jzon:write-value*
         (if (validation:string-is-nil-or-empty? (models::recipe-source recipe))
             nil
             (quri:uri-domain (quri:uri (models::recipe-source recipe)))))
        (jzon:write-key* :source-url)   (jzon:write-value* (models::recipe-source recipe))))))

(defun recipe-to-json (recipe-id title thumbnail-id ingredients source-domain source-url)
  "Takes the functions arg's and creates stringifies them to a JSON object.
`RECIPE-ID' doubles up as the 'id' for the Meilisearch database and as the
recipe's UUID, for the site's main database. The 'id' value, in the Meilisearch
database, is used as a way to update an entry in the Meilisearch database."
  (with-output-to-string (s)
    (jzon:with-writer* (:stream s :pretty t)
      (jzon:with-object*
        (jzon:write-key* :recipe-id)    (jzon:write-value* recipe-id)
        (jzon:write-key* :title)        (jzon:write-value* title)
        (jzon:write-key* :thumbnail-id) (jzon:write-value* thumbnail-id)
        (jzon:write-key* :ingredients)  (jzon:write-value* ingredients)
        (jzon:write-key* :source)       (jzon:write-value* source-domain)
        (jzon:write-key* :source-url)   (jzon:write-value* source-url)))))

        
