(in-package #:cl-user)
(defpackage #:larder.web.view
  (:use #:cl)
  (:import-from #:larder.web.config
                #:*template-directory*)
  (:import-from #:caveman2
                #:*response*
                #:response-headers)
  (:import-from #:djula
                #:add-template-directory
                #:compile-template*
                #:render-template*
                #:*djula-execute-package*)
  (:import-from #:datafly
                #:encode-json)
  (:export #:render
           #:render-json))
(in-package #:larder.web.view)

(djula:add-template-directory *template-directory*)

(defparameter *template-registry* (make-hash-table :test 'equal))

(defun render (template-path &optional env)
  (let ((template (gethash template-path *template-registry*)))
    (unless template
      (setf template (djula:compile-template* (princ-to-string template-path)))
      (setf (gethash template-path *template-registry*) template))
    (apply #'djula:render-template*
           template nil
           env)))

(defun render-json (object)
  (setf (getf (response-headers *response*) :content-type) "application/json")
  (encode-json object))


;;
;; Execute package definition

(defpackage #:larder.web.djula
  (:use #:cl
        #:app-constants
        #:storage)
  (:import-from #:larder.web.config
                #:config
                #:appenv
                #:developmentp
                #:productionp)
  (:import-from #:caveman2
                #:url-for))
;; Added after Caveman2 project generation. Needed for custom template and
;; filter code. See below...
(in-package #:larder.web.djula)

(setf djula:*djula-execute-package* (find-package :larder.web.djula))

#| Custom Filters and Template Code
================================================================================
These filters are to be used in the various HTML files in the templates/
directory. They are specific to this project and not formed as part of the
project generated, provided by Caveman2.
|#

(defun insert-snippet (snippet-name)
  (let ((filepath (merge-pathnames snippet-name app-constants:+storage-dir+)))
    (if (cl:probe-file filepath)
        (format nil "~a" (storage:open-text-file filepath))
        (format nil "<!-- ~a not found -->" snippet-name))))
