/**
Full Search
================================================================================
This file's main focus in providing the Meilisearch features to the
search.html template.

For more information on Meilisearch, use:

- https://docs.meilisearch.com/
- https://github.com/meilisearch/instant-meilisearch

Meilisearch provides an 'InstantSearch' add-on (plug-in?) which
links the Meilisearch instance you have running to this website (link above for
more information).
*/

/**
Variables defined in templates/search.html
================================================================================
They are declared there so the back-end can provide the data (via the use of a
Djula template).

- server     
- apiKey
- searchIndex
- searchURL

The rest of this file couldn't be included in the Djula template because of the
use of double-brackets. the Djula tempting engine uses '{{ }}' as a way to
denote 'back-end data' and it can't distinguish between Common Lisp back-end
code and JavaScript code. So, here we are with some code in
templates/search.html and this file.
**/ 

const search = instantsearch({
    indexName: searchIndex,
    searchClient: instantMeiliSearch(
	searchURL,
	apiKey,
	{
	    primaryKey: 'id',
	}
    )
});

search.addWidgets([
    instantsearch.widgets.searchBox({
        container: "#searchbox",
	placeholder: "Search the archive here...",
    }),
    instantsearch.widgets.configure({
	hitsPerPage: 50,
	snippetEllipsisText: "...",
	attributesToSnippet: ["title:100"]
    }),
    instantsearch.widgets.hits({
        container: "#hits",
        templates: {
            item: `
<div>
<a class="fe-search-hit" href="{{location.origin}}/view/recipe/{{recipe-id}}">
<img src="{{location.origin}}/storage/recipe-thumbnail/{{thumbnail-id}}"/>

<div class="fe-search-hit-text">

<span class="fe-search-hit-title">
{{#helpers.highlight}}{"attribute": "title"}{{/helpers.highlight}}
</span>

<div class="fe-search-hit-secondary">
{{#helpers.highlight}}{"attribute": "ingredients"}{{/helpers.highlight}}
</div>

</div>
</a>

<div class="{{source}}">
<a href="{{source-url}}">{{source}}</a>
<div>
`,}                
    }),
    instantsearch.widgets.pagination({
        container: "#pagination"
    }),
    instantsearch.widgets.stats({
        container: '#stats',
    })
]);

search.start();
